from discord.ext import commands
from jarvis.config import get_config
from jarvis.utils import get_prefix


class OwnerCog(commands.Cog):
    def __init__(self, bot):
        self.bot = bot
        self.admins = get_config().admins

    @commands.command(name="load", hidden=True)
    async def _load_cog(self, ctx, *, cog: str):
        info = await self.bot.application_info()
        if (
            ctx.message.author == info.owner
            or ctx.message.author.id in self.admins
        ):
            try:
                self.bot.load_extension(cog)
            except Exception as e:
                await ctx.send(
                    f"Failed to load new cog `{cog}`: {type(e).name} - {e}"
                )
            else:
                await ctx.send(f"Successfully loaded new cog `{cog}`")
        else:
            await ctx.send("I'm afraid I can't let you do that")

    @commands.command(name="unload", hidden=True)
    async def _unload_cog(self, ctx, *, cog: str):
        info = await self.bot.application_info()
        if (
            ctx.message.author == info.owner
            or ctx.message.author.id in self.admins
        ):
            try:
                self.bot.unload_extension(cog)
            except Exception as e:
                await ctx.send(
                    f"Failed to unload cog `{cog}` {type(e).__name__} - {e}"
                )
            else:
                await ctx.send(f"Successfully unloaded cog `{cog}`")
        else:
            await ctx.send("I'm afraid I can't let you do that")

    @commands.command(name="reload", hidden=True)
    async def _cog_reload(self, ctx, *, cog: str):
        info = await self.bot.application_info()
        if (
            ctx.message.author == info.owner
            or ctx.message.author.id in self.admins
        ):
            try:
                self.bot.unload_extension(cog)
                self.bot.load_extension(cog)
            except Exception as e:
                await ctx.send(
                    f"Failed to reload cog `{cog}` {type(e).__name__} - {e}"
                )
            else:
                await ctx.send(f"Successfully reloaded cog `{cog}`")
        else:
            await ctx.send("I'm afraid I can't let you do that")

    @commands.group(name="system", hidden=True, pass_context=True)
    async def _system(self, ctx):
        if ctx.invoked_subcommand is None:
            await ctx.send(
                f"Usage: `{ctx.message.content} <subcommand>`\n"
                + "Subcommands: `restart`, `update`"
            )

    @_system.command(name="restart", hidden=True)
    async def _restart(self, ctx):
        await ctx.send("Restarting core systems...")
        try:
            await self.bot.close()
        except RuntimeError:
            pass


def setup(bot):
    bot.add_cog(OwnerCog(bot))
